{% load static %}

<script
  src="{% static 'turbo/js/turbo.min.js' %}"
  defer
  data-turbo-track="reload"
></script>
<script
  src="{% static 'turbo/js/reconnecting-websocket.min.js' %}"
  defer
  data-turbo-track="reload"
></script>
<script
  src="{% static 'turbo/js/turbo-django.js' %}"
  defer
  data-turbo-track="reload"
></script>
